@@ -17,8 +17,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
+ * 02110-1301 USA.
  */
 
 
@@ -36,6 +36,7 @@
 #include "ODe_RDFWriter.h"
 #include "ODe_Main_Listener.h"
 #include "ODe_MetaDataWriter.h"
+#include "ODe_ThumbnailsWriter.h"
 #include "ODe_PicturesWriter.h"
 #include "ODe_SettingsWriter.h"
 
@@ -132,11 +133,13 @@ UT_Error IE_Exp_OpenDocument::copyToBuffer(PD_DocumentRange * pDocRange,UT_ByteB
     if( PD_DocumentRDFHandle outrdf = outDoc->getDocumentRDF() )
     {
 
-        std::list< std::string > xmlids;
+        std::set< std::string > xmlids;
         PD_DocumentRDFHandle inrdf = pDocRange->m_pDoc->getDocumentRDF();
         inrdf->addRelevantIDsForRange( xmlids, pDocRange );
+
         if( !xmlids.empty() )
         {
+            UT_DEBUGMSG(("MIQ: ODF export creating restricted RDF model xmlids.sz:%ld \n",(long)xmlids.size()));
             PD_RDFModelHandle subm = inrdf->createRestrictedModelForXMLIDs( xmlids );
             PD_DocumentRDFMutationHandle m = outrdf->createMutation();
             m->add( subm );
@@ -183,9 +186,9 @@ UT_Error IE_Exp_OpenDocument::copyToBuffer(PD_DocumentRange * pDocRange,UT_ByteB
     //
 
     GsfInput *  fData = gsf_input_stdio_new(szTempFileName,&err);
-    UT_sint32 siz = gsf_input_size(fData);
+    UT_DebugOnly<UT_sint32> siz = gsf_input_size(fData);
     const UT_Byte * pData = gsf_input_read(fData,gsf_input_size(fData),NULL);
-     UT_DEBUGMSG(("Writing %d bytes to clipboard \n",siz));
+    UT_DEBUGMSG(("Writing %d bytes to clipboard \n", (UT_sint32)siz));
     bufODT->append( pData, gsf_input_size(fData));
     
     delete pNewExp;
@@ -209,6 +212,9 @@ UT_Error IE_Exp_OpenDocument::_writeDocument(void)
     
 	UT_return_val_if_fail (getFp(), UT_ERROR);
 
+    PD_DocumentRDFHandle rdf = getDoc()->getDocumentRDF();
+    auxData.m_additionalRDF = rdf->createScratchModel();
+    
 	const std::string & prop = getProperty ("uncompressed");
 	
 	if (!prop.empty() && UT_parseBool (prop.c_str (), false))
@@ -253,6 +259,12 @@ UT_Error IE_Exp_OpenDocument::_writeDocument(void)
 		return UT_ERROR;
 	}
 
+	if (!ODe_ThumbnailsWriter::writeThumbnails(getDoc(), m_odt))
+	{
+		ODe_gsf_output_close(GSF_OUTPUT(m_odt));
+		return UT_ERROR;
+	}
+
     if (!ODe_SettingsWriter::writeSettings(getDoc(), m_odt))
 	{
 		ODe_gsf_output_close(GSF_OUTPUT(m_odt));
@@ -265,11 +277,6 @@ UT_Error IE_Exp_OpenDocument::_writeDocument(void)
 		return UT_ERROR;
 	}
 
-    if (!ODe_RDFWriter::writeRDF(getDoc(), m_odt))
-	{
-		ODe_gsf_output_close(GSF_OUTPUT(m_odt));
-		return UT_ERROR;
-	}
 
 	if (!ODe_ManifestWriter::writeManifest(getDoc(), m_odt))
 	{
@@ -347,6 +354,13 @@ UT_Error IE_Exp_OpenDocument::_writeDocument(void)
 		ODe_gsf_output_close(GSF_OUTPUT(m_odt));
 		return UT_ERROR;
 	}
+
+    // Write RDF.
+    if (!ODe_RDFWriter::writeRDF(getDoc(), m_odt, auxData.m_additionalRDF ))
+	{
+		ODe_gsf_output_close(GSF_OUTPUT(m_odt));
+		return UT_ERROR;
+	}
     
     // Write content and styles
         
